activity.setTitle("Created by syed wajeeh ul hassan naqvi ")
require "import"
import "android.widget.*"
import "android.view.*"
import "android.content.*"
import "android.text.*"
import "android.net.Uri"
import "android.speech.tts.TextToSpeech"
import "android.app.*"
import "cjson"

-- Global variables
local tts
local links = {}
local prefs = activity.getSharedPreferences("SocialLinksApp", Context.MODE_PRIVATE)

-- Load saved links
function loadLinks()
    local savedLinks = prefs.getString("links", "")
    if savedLinks ~= "" then
        pcall(function()
            links = cjson.decode(savedLinks)
        end)
    end
    
    if not links or #links == 0 then
        links = {
            {
                title = "Example Link",
                url = "https://example.com",
                date = "2024-01-01"
            }
        }
    end
end

-- Save links to storage
function saveLinks()
    local editor = prefs.edit()
    pcall(function()
        local jsonLinks = cjson.encode(links)
        editor.putString("links", jsonLinks)
        editor.commit()
    end)
end

-- Function to copy text to clipboard
function copyToClipboard(text)
    activity.getSystemService(Context.CLIPBOARD_SERVICE).setText(text)
    Toast.makeText(activity, "Copied to clipboard!", Toast.LENGTH_SHORT).show()
end

-- Function to share link
function shareLink(title, url)
    local intent = Intent(Intent.ACTION_SEND)
    intent.setType("text/plain")
    intent.putExtra(Intent.EXTRA_TEXT, url)
    activity.startActivity(Intent.createChooser(intent, "Share " .. title))
end

-- Function to open link
function openLink(url)
    local intent = Intent(Intent.ACTION_VIEW)
    intent.setData(Uri.parse(url))
    activity.startActivity(intent)
end

-- Update ListView
function updateList()
    data = {}
    for k,v in ipairs(links) do
        table.insert(data, v.title .. "\n" .. v.date)
    end
    adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, String(data))
    list.setAdapter(adapter)
    saveLinks()
end

-- Create main layout
activity.setContentView(loadlayout({
    LinearLayout,
    orientation="vertical",
    {
        Button,
        id="addButton",
        text="Add New Link",
        layout_width="match_parent",
    },
    {
        ListView,
        id="list",
        layout_width="match_parent",
        layout_height="match_parent",
    },
}))

-- Add new link button click
addButton.onClick = function()
    local dialog = Dialog(activity)
    dialog.setContentView(loadlayout({
        LinearLayout,
        orientation="vertical",
        padding="16dp",
        {
            EditText,
            id="title",
            hint="Enter Link Name",
            layout_width="match_parent",
        },
        {
            EditText,
            id="url",
            hint="Enter or Paste Link Here",
            layout_width="match_parent",
        },
        {
            Button,
            text="Save",
            layout_width="match_parent",
            onClick=function()
                if title.Text ~= "" and url.Text ~= "" then
                    table.insert(links, 1, {
                        title = title.Text,
                        url = url.Text,
                        date = os.date("%Y-%m-%d")
                    })
                    updateList()
                    dialog.dismiss()
                    Toast.makeText(activity, "Link saved!", Toast.LENGTH_SHORT).show()
                else
                    Toast.makeText(activity, "Please fill all fields", Toast.LENGTH_SHORT).show()
                end
            end
        }
    }))
    dialog.show()
end

-- Handle link item click
list.onItemClick = function(parent, view, position, id)
    local link = links[position + 1]
    local dialog = Dialog(activity)
    dialog.setContentView(loadlayout({
        LinearLayout,
        orientation="vertical",
        padding="16dp",
        {
            TextView,
            text=link.title,
            textSize="20sp",
            layout_width="match_parent",
        },
        {
            TextView,
            text=link.url,
            layout_width="match_parent",
            layout_marginTop="8dp",
        },
        {
            LinearLayout,
            orientation="horizontal",
            layout_width="match_parent",
            layout_marginTop="16dp",
            {
                Button,
                text="Copy",
                layout_weight="1",
                onClick=function()
                    copyToClipboard(link.url)
                end
            },
            {
                Button,
                text="Share",
                layout_weight="1",
                onClick=function()
                    shareLink(link.title, link.url)
                end
            },
            {
                Button,
                text="Open",
                layout_weight="1",
                onClick=function()
                    openLink(link.url)
                end
            },
            {
                Button,
                text="Close",
                layout_weight="1",
                onClick=function()
                    dialog.dismiss()
                end
            }
        }
    }))
    dialog.show()
end

-- Handle long press for edit/delete options
list.onItemLongClick = function(parent, view, position, id)
    local link = links[position + 1]
    local items = {"Edit", "Delete"}
    
    AlertDialog.Builder(activity)
    .setTitle("Options")
    .setItems(items, function(dialog, which)
        if which == 0 then -- Edit
            local editDialog = Dialog(activity)
            editDialog.setContentView(loadlayout({
                LinearLayout,
                orientation="vertical",
                padding="16dp",
                {
                    EditText,
                    id="editTitle",
                    text=link.title,
                    layout_width="match_parent",
                },
                {
                    EditText,
                    id="editUrl",
                    text=link.url,
                    layout_width="match_parent",
                },
                {
                    Button,
                    text="Save",
                    layout_width="match_parent",
                    onClick=function()
                        if editTitle.Text ~= "" and editUrl.Text ~= "" then
                            links[position + 1] = {
                                title = editTitle.Text,
                                url = editUrl.Text,
                                date = os.date("%Y-%m-%d")
                            }
                            updateList()
                            editDialog.dismiss()
                            Toast.makeText(activity, "Link updated!", Toast.LENGTH_SHORT).show()
                        else
                            Toast.makeText(activity, "Please fill all fields", Toast.LENGTH_SHORT).show()
                        end
                    end
                }
            }))
            editDialog.show()
            
        elseif which == 1 then -- Delete
            AlertDialog.Builder(activity)
            .setTitle("Confirm Delete")
            .setMessage("Are you sure you want to delete this link?")
            .setPositiveButton("Yes", function()
                table.remove(links, position + 1)
                updateList()
                Toast.makeText(activity, "Link deleted!", Toast.LENGTH_SHORT).show()
            end)
            .setNegativeButton("No", nil)
            .show()
        end
    end)
    .show()
    return true
end

-- Initialize
loadLinks()
updateList()